<?php

namespace Acms\Services\Session;

class Engine
{
    protected $sessionNmae = '';

    protected $lifetime = 0;

    protected $path = '';

    protected $domain = '';

    protected $secure = false;

    protected $httpOnly = false;

    protected $storage = [];

    protected $storageName = 'acms_storage';

    /**
     * Constructor
     */
    public function __construct($sessionName, $config)
    {
        $this->sessionNmae = $sessionName;
        foreach ($config as $key => $val) {
            if (!property_exists($this, $key)) {
                continue;
            }
            $this->$key = $val;
        }
        if (defined('PHP_SESSION_USE_DB') && PHP_SESSION_USE_DB) {
            $this->useDatabase();
        }
        $this->start(); // セッションスタート
        $this->storage = isset($_SESSION[$this->storageName]) ? acmsUnserialize($_SESSION[$this->storageName]) : []; // セッション変数から，前回の状態を取得
    }

    public function handle()
    {
        return $this;
    }

    public function getSessionId()
    {
        return session_id();
    }

    /**
     * Regenerate session.
     */
    public function regenerate()
    {
        session_regenerate_id(true);
    }

    /**
     * Save
     *
     * @return void
     */
    public function save()
    {
        $_SESSION[$this->storageName] = acmsSerialize($this->storage);
    }

    /**
     * Get
     *
     * @param string $key
     *
     * @return bool
     */
    public function get($key)
    {
        return isset($this->storage[$key]) ? $this->storage[$key] : false;
    }

    /**
     * Set
     *
     * @param string $key
     * @param mixed $val
     *
     * @return void
     */
    public function set($key, $val)
    {
        $this->storage[$key] = $val;
    }

    /**
     * Delete
     *
     * @param string $key
     *
     * @return void
     */
    public function delete($key)
    {
        unset($this->storage[$key]);
    }

    /**
     * Clear
     *
     * @return void
     */
    public function clear()
    {
        $this->storage = null;
        $this->save();
    }

    /**
     * Session start.
     *
     * @return void
     */
    protected function start()
    {
        session_name($this->sessionNmae);
        session_set_cookie_params($this->lifetime, $this->path, $this->domain, $this->secure, $this->httpOnly);
        session_cache_limiter(null); // HTTPヘッダーは独自で設定する
        session_start();
    }

    /**
     * Destroy session.
     */
    public function destroy()
    {
        $_SESSION = [];
        $params = session_get_cookie_params();
        setcookie(session_name(), '', time() - 42000, $params["path"], $params["domain"], $params["secure"], $params["httponly"]);
        session_destroy();
    }

    /**
     * セッションデータをCookieではなくDBに保存
     */
    protected function useDatabase()
    {
        $handler = new DatabaseHandler();
        if (session_set_save_handler(
            array($handler, 'open'),
            array($handler, 'close'),
            array($handler, 'read'),
            array($handler, 'write'),
            array($handler, 'destroy'),
            array($handler, 'gc')
        )) {
            register_shutdown_function('session_write_close');
        }
    }
}
